@@ -4,11 +4,12 @@ import requests
 from flask import Flask, current_app, g
 from flask.ext.principal import identity_loaded
 from flask.ext.sqlalchemy import SQLAlchemy
+from flask.ext.wtf.csrf import CsrfProtect
 
 from .sqlstats import DB_STATS
 
 
-__version__ = '0.3'
+__version__ = '0.4'
 
 
 requests_session = requests.Session()
@@ -25,6 +26,9 @@ from . import models
 from .auth import models
 
 
+csrf = CsrfProtect()
+
+
 def create_app(**kwargs):
     app = Flask('evesrp', **kwargs)
     app.config.from_object('evesrp.default_config')
@@ -40,11 +44,13 @@ def create_app(**kwargs):
     from .auth import principal
     principal.init_app(app)
 
+    csrf.init_app(app)
+
     from .views import index, divisions, login, requests, api
     app.add_url_rule(rule='/', view_func=index)
     app.register_blueprint(divisions.blueprint, url_prefix='/divisions')
     app.register_blueprint(login.blueprint)
-    app.register_blueprint(requests.blueprint, url_prefix='/requests')
+    app.register_blueprint(requests.blueprint)
     app.register_blueprint(api.api, url_prefix='/api')
     app.register_blueprint(api.filters, url_prefix='/api/filter')
 
@@ -57,6 +63,7 @@ def create_app(**kwargs):
     app.before_first_request(_copy_config_to_authmethods)
     app.before_first_request(_config_requests_session)
     app.before_first_request(_config_killmails)
+    app.before_first_request(_copy_url_converter_config)
 
     return app
 
@@ -72,6 +79,20 @@ def _copy_config_to_authmethods():
     current_app.auth_methods = current_app.config['AUTH_METHODS']
 
 
+# Request detail URL setup
+def _copy_url_converter_config():
+    current_app.ship_urls = {}
+    ship_transformers = current_app.config.get('SRP_SHIP_URL_TRANSFORMERS')
+    if ship_transformers is not None:
+        for transformer in ship_transformers:
+            current_app.ship_urls[transformer.name] = transformer
+    current_app.pilot_urls = {}
+    pilot_transformers = current_app.config.get('SRP_PILOT_URL_TRANSFORMERS')
+    if pilot_transformers is not None:
+        for transformer in pilot_transformers:
+            current_app.pilot_urls[transformer.name] = transformer
+
+
 # Requests session setup
 def _config_requests_session():
     try:
