@@ -172,6 +172,11 @@ class Note(db.Model, AutoID, Timestamped):
     noter = db.relationship(User, back_populates='notes_made',
             foreign_keys=[noter_id])
 
+    def __init__(self, user, noter, note):
+        self.user = user
+        self.noter = noter
+        self.content = note
+
 
 class Pilot(db.Model, AutoID):
     """Represents an in-game character."""
@@ -255,6 +260,7 @@ class Permission(db.Model, AutoID):
         return ("{x.__class__.__name__}('{x.permission}', {x.entity}, "
                "{x.division})").format(x=self)
 
+
 class Division(db.Model, AutoID):
     """A reimbursement division.
 
@@ -270,6 +276,13 @@ class Division(db.Model, AutoID):
     division_permissions = db.relationship(Permission, collection_class=set,
             back_populates='division')
 
+    #: :py:class:`Request` s filed under this division.
+    requests = db.relationship(Request, back_populates='division')
+
+    ship_transformer = db.Column(db.PickleType, nullable=True, default=None)
+
+    pilot_transformer = db.Column(db.PickleType, nullable=True, default=None)
+
     @property
     def permissions(self):
         """The permissions objects for this division, mapped via their
@@ -282,9 +295,6 @@ class Division(db.Model, AutoID):
                 return set(filter(lambda x: x.permission == key, self.perms))
         return _PermProxy(self.division_permissions)
 
-    #: :py:class:`Request` s filed under this division.
-    requests = db.relationship(Request, back_populates='division')
-
     def __init__(self, name):
         self.name = name
 
