@@ -1,8 +1,9 @@
 from collections import OrderedDict
 from itertools import groupby
+import re
 
 from flask import render_template, abort, url_for, flash, Markup, request,\
-    redirect, current_app, Blueprint
+    redirect, current_app, Blueprint, Markup
 from flask.views import View
 from flask.ext.login import login_required, current_user
 from flask.ext.wtf import Form
@@ -15,7 +16,7 @@ from .. import db
 from ..models import Request, Modifier, Action
 from ..auth import SubmitRequestsPermission, ReviewRequestsPermission, \
         PayoutRequestsPermission, admin_permission
-from ..auth.models import Division, Pilot, Permission, User, Group
+from ..auth.models import Division, Pilot, Permission, User, Group, Note
 
 
 blueprint = Blueprint('requests', __name__)
@@ -67,14 +68,14 @@ class RequestListing(View):
         )
 
 
-class SubmittedRequestListing(RequestListing):
-    """A requests listing with a button for submitting requests at the bottom.
+class PersonalRequests(RequestListing):
+    """Shows a list of all personally submitted requests and divisions they
+    have permissions in.
 
-    It will show all requests the current user has submitted. The button links
-    to :py:func:`submit_request`.
+    It will show all requests the current user has submitted.
     """
 
-    template = 'list_submit.html'
+    template = 'personal.html'
 
     def requests(self, division_id=None):
         requests = Request.query\
@@ -126,6 +127,23 @@ class PermissionRequestListing(RequestListing):
         return requests
 
 
+class PayoutListing(PermissionRequestListing):
+
+    template = 'payout.html'
+
+    def __init__(self):
+        # Just a special case of PermissionRequestListing
+        super(PayoutListing, self).__init__(('pay',), ('approved',))
+
+    def dispatch_request(self, division_id=None, page=1):
+        """Returns the response to requests.
+
+        Part of the :py:class:`flask.views.View` interface.
+        """
+        pager = self.requests(division_id).paginate(page, per_page=20)
+        return render_template(self.template, form=ActionForm(), pager=pager)
+
+
 def register_perm_request_listing(app, endpoint, path, permissions, statuses):
     """Utility function for creating :py:class:`PermissionRequestListing`
     views.
@@ -149,17 +167,28 @@ def register_perm_request_listing(app, endpoint, path, permissions, statuses):
 
 @blueprint.record
 def register_class_views(state):
+    try:
+        prefixes = state.app.request_prefixes
+    except AttributeError:
+        prefixes = []
+        state.app.request_prefixes = prefixes
+    prefixes.append(state.url_prefix if state.url_prefix is not None else '')
     """Register class based views onto the requests blueprint."""
-    submit_view = SubmittedRequestListing.as_view('list_submit_requests')
-    state.add_url_rule('/submit/', view_func=submit_view)
-    state.add_url_rule('/submit/<int:page>/', view_func=submit_view)
-    state.add_url_rule('/submit/<int:page>/<int:division_id>', view_func=submit_view)
-    register_perm_request_listing(state, 'list_review_requests',
-            '/review/', ('review',), ('evaluating', 'incomplete', 'approved'))
-    register_perm_request_listing(state, 'list_approved_requests',
-            '/pay/', ('pay',), ('approved',))
+    personal_view = PersonalRequests.as_view('personal_requests')
+    state.add_url_rule('/personal/', view_func=personal_view)
+    state.add_url_rule('/personal/<int:page>/', view_func=personal_view)
+    state.add_url_rule('/personal/<int:page>/<int:division_id>',
+            view_func=personal_view)
+    payout_view = PayoutListing.as_view('list_approved_requests')
+    payout_url_stub = '/pay/'
+    state.add_url_rule(payout_url_stub, view_func=payout_view)
+    state.add_url_rule(payout_url_stub + '<int:page>/', view_func=payout_view)
+    state.add_url_rule(payout_url_stub + '<int:page>/<int:division_id>/',
+            view_func=payout_view)
+    register_perm_request_listing(state, 'list_pending_requests',
+            '/pending/', ('review',), ('evaluating', 'incomplete', 'approved'))
     register_perm_request_listing(state, 'list_completed_requests',
-            '/complete/', ('review', 'pay'), ('rejected', 'paid'))
+            '/completed/', ('review', 'pay'), ('rejected', 'paid'))
 
 
 class ValidKillmail(URL):
@@ -207,6 +236,19 @@ class RequestForm(Form):
             raise ValidationError([str(e) for e in failures])
 
 
+def submit_divisions(user):
+    submit_perms = user.permissions\
+            .filter_by(permission='submit')\
+            .subquery()
+    divisions = db.session.query(Division).join(submit_perms)\
+            .order_by(Division.name)
+    # Remove duplicates and sort divisions by name
+    choices = []
+    for name, group in groupby(divisions, lambda d: d.name):
+        choices.append((next(group).id, name))
+    return choices
+
+
 @blueprint.route('/add/', methods=['GET', 'POST'])
 @login_required
 def submit_request():
@@ -222,17 +264,7 @@ def submit_request():
         abort(403)
     form = RequestForm()
     # Create a list of divisions this user can submit to
-    submit_perms = current_user.permissions\
-            .filter_by(permission='submit')\
-            .subquery()
-    divisions = db.session.query(Division).join(submit_perms)
-    # Remove duplicates and sort divisions by name
-    keyfunc = lambda d: d.name
-    divisions = sorted(divisions, key=keyfunc)
-    choices = []
-    for name, group in groupby(divisions, keyfunc):
-        choices.append((next(group).id, name))
-    form.division.choices = choices
+    form.division.choices = submit_divisions(current_user)
     if form.validate_on_submit():
         mail = form.killmail
         # Prevent submitting other people's killmails
@@ -457,3 +489,106 @@ def request_detail(request_id):
             payout_form=PayoutForm(formdata=None),
             action_form=ActionForm(formdata=None),
             void_form=VoidModifierForm(formdata=None))
+
+
+class DetailForm(Form):
+    details = TextAreaField('Details', validators=[InputRequired()])
+    submit = SubmitField('Submit')
+
+
+@blueprint.route('/<int:request_id>/update/', methods=['GET', 'POST'])
+@login_required
+def request_detail_update(request_id):
+    srp_request = Request.query.get_or_404(request_id)
+    # Only the submitter can change the details, and only if it isn't finalized
+    if current_user != srp_request.submitter or srp_request.finalized:
+        abort(403)
+    form = DetailForm()
+    if form.validate_on_submit():
+        archive_note = 'Old Details: ' + srp_request.details
+        archive_action = Action(srp_request, current_user, archive_note)
+        archive_action.type_ = 'evaluating'
+        srp_request.details = form.details.data
+        db.session.commit()
+        return redirect(url_for('.request_detail', request_id=request_id))
+    form.details.data = srp_request.details
+    return render_template('form.html', form=form)
+
+
+class DivisionChange(Form):
+    division = SelectField('Divisions', coerce=int)
+    submit = SubmitField('Submit')
+
+
+@blueprint.route('/<int:request_id>/division', methods=['GET', 'POST'])
+@login_required
+def request_change_division(request_id):
+    srp_request = Request.query.get_or_404(request_id)
+    review_perm = ReviewRequestsPermission(srp_request)
+    if not review_perm.can() and \
+            current_user != srp_request.submitter or \
+            srp_request.finalized:
+        abort(403)
+    form = DivisionChange()
+    form.division.choices = submit_divisions(srp_request.submitter)
+    if form.validate_on_submit():
+        new_division = Division.query.get(form.division.data)
+        archive_note = "Moving from division '{}' to division '{}'.".format(
+                srp_request.division.name,
+                new_division.name)
+        archive_action = Action(srp_request, current_user, archive_note)
+        archive_action.type_ = 'evaluating'
+        srp_request.division = new_division
+        db.session.commit()
+        return redirect(url_for('.request_detail', request_id=request_id))
+    form.division.data = srp_request.division.id
+    return render_template('form.html', form=form)
+
+
+@blueprint.route('/notes/<int:user_id>/', methods=['GET', 'POST'])
+@login_required
+def user_notes(user_id):
+    if not current_user.has_permission(('review', 'pay')):
+        abort(403)
+    user = User.query.get_or_404(user_id)
+    return render_template('notes.html', user=user)
+
+
+class AddNote(Form):
+    note = TextAreaField('Note',
+            description=("If you have something like '#{Kill ID}', it will be "
+                         "linkified to the corresponding request "
+                         "(if it exists)."),
+            validators=[InputRequired()])
+    submit = SubmitField('Submit')
+
+
+killmail_re = re.compile(r'#(\d+)')
+
+
+@blueprint.route('/notes/<int:user_id>/add/', methods=['GET', 'POST'])
+@login_required
+def add_user_note(user_id):
+    if not current_user.has_permission(('review', 'pay')):
+        abort(403)
+    user = User.query.get_or_404(user_id)
+    form = AddNote()
+    if form.validate_on_submit():
+        # Linkify killmail IDs
+        note_content = Markup.escape(form.note.data)
+        for match in killmail_re.findall(note_content):
+            kill_id = int(match)
+            srp_request = db.session.query(Request.id).filter_by(id=kill_id)
+            print(kill_id, srp_request.all())
+            if db.session.query(srp_request.exists()):
+                link = '<a href="{url}">#{kill_id}</a>'.format(
+                        url=url_for('.request_detail', request_id=kill_id),
+                        kill_id=kill_id)
+                link = Markup(link)
+                note_content = note_content.replace('#' + match, link)
+        # Create the note
+        note = Note(user, current_user, note_content)
+        db.session.add(note)
+        db.session.commit()
+        return redirect(url_for('.user_notes', user_id=user_id))
+    return render_template('form.html', form=form)
